import React, { useState } from "react";
import axios from "axios";
import styles from "./CreateUsersStyles.module.css";
// import uploadFile from '~/backend/service/uploadFileService';
// import * as XLSX from 'xlsx';

const CreateUsers = () => {
  const [option, setOption] = useState("");
  const [personInfo, setPersonInfo] = useState({});
  const [degrees, setDegrees] = useState([]);
  const [addresses, setAddresses] = useState([]);
  const [contacts, setContacts] = useState([]);
  const [involvements, setInvolvements] = useState([]);
  const [userInfo, setUserInfo] = useState({});
  const [file, setFile] = useState(null);

  // // Function to handle file selection
  // const handleFileChange = (event) => {
  //   const selectedFile = event.target.files[0]; // Get the first file
  //   setFile(selectedFile); // Update the file state
  // };

  // // Function to be called when the upload button is clicked
  // const handleUploadClick = () => {
  //   if (file) {
  //     uploadFile(file); // Call the test function with the selected file
  //   } else {
  //     console.log("No file selected");
  //   }
  // };

  const trimValues = (obj) => {
    return Object.fromEntries(
      Object.entries(obj).map(([key, value]) => [
        key,
        typeof value === "string" ? value.trim() : value,
      ])
    );
  };

  const trimArrayValues = (array) => {
    return array.map((item) => trimValues(item));
  };

  const handleInputChange = (setter) => (e) => {
    const { name, value } = e.target;
    setter((prev) => ({ ...prev, [name]: value.trim() }));
  };

  const handleArrayChange = (setter) => (index, e) => {
    const { name, value } = e.target;
    setter((prev) =>
      prev.map((item, i) =>
        i === index ? { ...item, [name]: value.trim() } : item
      )
    );
  };

  const addArrayEntry = (setter, fields) => () => {
    setter((prev) => [...prev, fields]);
  };

  const removeArrayEntry = (setter) => (index) => {
    setter((prev) => prev.filter((_, i) => i !== index));
  };

  const handleSubmit = async (e) => {
    e.preventDefault();

    const token = localStorage.getItem("jwtToken");

    if (!token) {
      console.log("No token found");
      return;
    }

    // Trim whitespace for each section of payload
    const payload = {
      personInfo: trimValues(personInfo),
      degrees: trimArrayValues(degrees),
      addresses: trimArrayValues(addresses),
      contacts: trimArrayValues(contacts),
      involvements: involvements.map((inv) => inv.trim()), // trim for each involvement entry
      userInfo: trimValues(userInfo),
    };

    try {
      const response = await axios.post(
        `${process.env.REACT_APP_API_URL}/person/create`,
        payload,
        {
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
        }
      );

      if (response.status === 201) {
        alert("User created successfully!");
        setPersonInfo({});
        setDegrees([]);
        setAddresses([]);
        setContacts([]);
        setInvolvements([]);
        setUserInfo({});
      } else {
        alert("Unexpected response status: " + response.status);
      }
    } catch (error) {
      console.error("Error:", error);
      alert("Error creating user. Please check your network connection.");
    }
  };

  const handleFileChange = (e) => {
    setFile(e.target.files[0]); // Store the selected file
  };

  // Handle the form submission (uploading the file)
  const handleUploadFileSubmit = async (e) => {
    e.preventDefault();

    if (!file) {
      alert("Please select a file before submitting.");
      return;
    }

    // Prepare FormData to send the file
    const formData = new FormData();
    formData.append("file", file); // Append the file with the 'file' key

    try {
      const response = await axios.post(
        `${process.env.REACT_APP_API_URL}/people/create-many`, // The endpoint you defined on your server
        formData,
        {
          headers: {
            "Content-Type": "multipart/form-data", // Let axios handle the correct Content-Type
          },
        }
      );

      if (response.status === 201) {
        alert("File uploaded and processed successfully!");
        setFile(null); // Optionally reset the file input after successful upload
      } else {
        alert("Unexpected response status: " + response.status);
      }
    } catch (error) {
      console.error("Error uploading file:", error);
      alert("Error uploading file. Please check your network connection.");
    }
  };

  const renderContent = () => {
    if (option === "single") {
      return (
        <form className={styles.formContainer} onSubmit={handleSubmit}>
          <h3>Create a Single User</h3>

          {/* Person Info */}
          <fieldset className={styles.fieldset}>
            <legend>Person Info</legend>
            {Object.entries({
              firstName: "First Name",
              lastName: "Last Name",
              middleName: "Middle Name",
              maidenName: "Maiden Name",
              suffix: "Suffix",
              nickName: "Nick Name",
              techAlumniChapter: "Tech Alumni Chapter",
              classYear: "Class Year",
              gradYear: "Graduation Year",
              gradSemester: "Graduation Semester",
              gender: "Gender",
            }).map(([name, label]) => (
              <label key={name} className={styles.label}>
                {label}:
                <input
                  type="text"
                  name={name}
                  value={personInfo[name] || ""}
                  onChange={handleInputChange(setPersonInfo)}
                />
              </label>
            ))}
          </fieldset>

          {/* Degrees */}
          <fieldset className={styles.fieldset}>
            <legend>Degrees</legend>
            {degrees.map((degree, index) => (
              <div key={index}>
                {Object.entries({
                  degreeType: "Degree Type",
                  degreeYear: "Degree Year",
                  degreeDescription: "Degree Description",
                  degreeDepartment: "Degree Department",
                  degreeCollege: "Degree College",
                }).map(([name, label]) => (
                  <label key={name} className={styles.label}>
                    {label}:
                    <input
                      type="text"
                      name={name}
                      value={degree[name] || ""}
                      onChange={(e) => handleArrayChange(setDegrees)(index, e)}
                    />
                  </label>
                ))}
                <button
                  type="button"
                  className={`${styles.button} ${styles.remove}`}
                  onClick={() => removeArrayEntry(setDegrees)(index)}
                >
                  Remove Degree
                </button>
              </div>
            ))}
            <button
              type="button"
              onClick={addArrayEntry(setDegrees, {
                degreeType: "",
                degreeYear: "",
                degreeDescription: "",
                degreeDepartment: "",
                degreeCollege: "",
              })}
            >
              Add Degree
            </button>
          </fieldset>

          {/* Addresses */}
          <fieldset className={styles.fieldset}>
            <legend>Addresses</legend>
            {addresses.map((address, index) => (
              <div key={index}>
                {Object.entries({
                  address1: "Address Line 1",
                  address2: "Address Line 2",
                  city: "City",
                  state: "State",
                  country: "Country",
                  zipCode: "Zip Code",
                  preferredAddress: "Preferred Address",
                }).map(([name, label]) => (
                  <label key={name} className={styles.label}>
                    {label}:
                    <input
                      type={name === "preferredAddress" ? "checkbox" : "text"}
                      name={name}
                      value={address[name] || ""}
                      onChange={(e) =>
                        handleArrayChange(setAddresses)(index, e)
                      }
                    />
                  </label>
                ))}
                <button
                  type="button"
                  className={`${styles.button} ${styles.remove}`}
                  onClick={() => removeArrayEntry(setAddresses)(index)}
                >
                  Remove Address
                </button>
              </div>
            ))}
            <button
              type="button"
              onClick={addArrayEntry(setAddresses, {
                address1: "",
                address2: "",
                city: "",
                state: "",
                country: "",
                zipCode: "",
                preferredAddress: false,
              })}
            >
              Add Address
            </button>
          </fieldset>

          {/* Contacts */}
          <fieldset className={styles.fieldset}>
            <legend>Contacts</legend>
            {contacts.map((contact, index) => (
              <div key={index}>
                {Object.entries({
                  contactNumber: "Contact Number",
                  contactType: "Contact Type",
                  preferredContact: "Preferred Contact",
                }).map(([name, label]) => (
                  <label key={name} className={styles.label}>
                    {label}:
                    <input
                      type={name === "preferredContact" ? "checkbox" : "text"}
                      name={name}
                      value={contact[name] || ""}
                      onChange={(e) => handleArrayChange(setContacts)(index, e)}
                    />
                  </label>
                ))}
                <button
                  type="button"
                  className={`${styles.button} ${styles.remove}`}
                  onClick={() => removeArrayEntry(setContacts)(index)}
                >
                  Remove Contact
                </button>
              </div>
            ))}
            <button
              type="button"
              onClick={addArrayEntry(setContacts, {
                contactNumber: "",
                contactType: "",
                preferredContact: false,
              })}
            >
              Add Contact
            </button>
          </fieldset>

          {/* Involvements */}
          <fieldset className={styles.fieldset}>
            <legend>Involvements</legend>
            {involvements.map((involvement, index) => (
              <div key={index}>
                <input
                  type="text"
                  value={involvement}
                  onChange={(e) =>
                    setInvolvements((prev) =>
                      prev.map((item, i) =>
                        i === index ? e.target.value : item
                      )
                    )
                  }
                />
                <button
                  type="button"
                  className={`${styles.button} ${styles.remove}`}
                  onClick={() => removeArrayEntry(setInvolvements)(index)}
                >
                  Remove Involvement
                </button>
              </div>
            ))}
            <button
              type="button"
              onClick={() => setInvolvements((prev) => [...prev, ""])}
            >
              Add Involvement
            </button>
          </fieldset>

          {/* User Info */}
          <fieldset className={styles.fieldset}>
            <legend>User Info</legend>
            {Object.entries({
              pid: "PID",
              roleId: "Role ID",
              roleInfo: "Role Info",
            }).map(([name, label]) => (
              <label key={name} className={styles.label}>
                {label}:
                <input
                  type="text"
                  name={name}
                  value={userInfo[name] || ""}
                  onChange={handleInputChange(setUserInfo)}
                />
              </label>
            ))}
          </fieldset>

          <button type="submit" className={styles.submitButton}>
            Submit
          </button>
        </form>
      );
    } else if (option === "csv") {
      return (
        <div className={styles.formContainer}>
          <h3>Upload Excel File</h3>
          <input type="file" accept=".xlsx" onChange={handleFileChange} />
          <button
            type="submit"
            className={styles.submitButton}
            onClick={handleUploadFileSubmit}
          >
            Upload
          </button>
        </div>
      );
    } else {
      return <p>Please select an option to proceed.</p>;
    }
  };

  return (
    <section className={styles.container}>
      <h2>Create Users</h2>
      <p>Select an option to proceed:</p>
      <div className={styles.options}>
        <button
          onClick={() => setOption("single")}
          className={styles.optionButton}
        >
          Create Single User
        </button>
        <button
          onClick={() => setOption("xlsx")}
          className={styles.optionButton}
        >
          Upload XLSX
        </button>
      </div>
      <div style={{ marginTop: "1rem" }}>{renderContent()}</div>
    </section>
  );
};

export default CreateUsers;
